@@ -14,23 +14,18 @@
 # limitations under the License.
 
 
-# FIXME: https://bugs.launchpad.net/climate/+bug/1300132
-# from oslo_log import log as logging
 from oslo_utils import uuidutils
-import six
 import sqlalchemy as sa
 from sqlalchemy.dialects.mysql import MEDIUMTEXT
 from sqlalchemy.orm import relationship
 
 from blazar.db.sqlalchemy import model_base as mb
-# FIXME: https://bugs.launchpad.net/climate/+bug/1300132
-# LOG = logging.getLogger(__name__)
 
 # Helpers
 
 
 def _generate_unicode_uuid():
-    return six.text_type(uuidutils.generate_uuid())
+    return str(uuidutils.generate_uuid())
 
 
 def MediumText():
@@ -88,28 +83,38 @@ class Reservation(mb.BlazarBase):
                                   server_default=sa.false())
     resources_changed = sa.Column(sa.Boolean, nullable=False,
                                   server_default=sa.false())
-    instance_reservations = relationship('InstanceReservations',
-                                         uselist=False,
-                                         cascade='all,delete',
-                                         backref='reservation',
-                                         lazy='joined')
-    computehost_reservations = relationship('ComputeHostReservation',
-                                            uselist=False,
-                                            cascade="all,delete",
-                                            backref='reservation',
-                                            lazy='joined')
-    computehost_allocations = relationship('ComputeHostAllocation',
+    instance_reservation = relationship('InstanceReservations',
+                                        uselist=False,
+                                        cascade='all,delete',
+                                        backref='reservation',
+                                        lazy='joined')
+    computehost_reservation = relationship('ComputeHostReservation',
                                            uselist=False,
                                            cascade="all,delete",
                                            backref='reservation',
                                            lazy='joined')
+    computehost_allocations = relationship('ComputeHostAllocation',
+                                           uselist=True,
+                                           cascade="all,delete",
+                                           backref='reservation',
+                                           lazy='joined')
+    floatingip_reservation = relationship('FloatingIPReservation',
+                                          uselist=False,
+                                          cascade="all,delete",
+                                          backref='reservation',
+                                          lazy='joined')
+    floatingip_allocations = relationship('FloatingIPAllocation',
+                                          uselist=True,
+                                          cascade="all,delete",
+                                          backref='reservation',
+                                          lazy='joined')
 
     def to_dict(self):
         d = super(Reservation, self).to_dict()
 
-        if self.computehost_reservations:
+        if self.computehost_reservation:
 
-            res = self.computehost_reservations.to_dict()
+            res = self.computehost_reservation.to_dict()
             d['hypervisor_properties'] = res['hypervisor_properties']
             d['resource_properties'] = res['resource_properties']
             d['before_end'] = res['before_end']
@@ -119,16 +124,18 @@ class Reservation(mb.BlazarBase):
                     minMax = res['count_range'].split('-', 1)
                     (d['min'], d['max']) = map(int, minMax)
                 except ValueError:
-                    # FIXME: https://bugs.launchpad.net/climate/+bug/1300132
-                    # LOG.error(
-                    # "Invalid Range: {0}".format(res['count_range']))
                     e = "Invalid count range: {0}".format(res['count_range'])
                     raise RuntimeError(e)
 
-        if self.instance_reservations:
+        if self.instance_reservation:
             ir_keys = ['vcpus', 'memory_mb', 'disk_gb', 'amount', 'affinity',
-                       'flavor_id', 'aggregate_id', 'server_group_id']
-            d.update(self.instance_reservations.to_dict(include=ir_keys))
+                       'flavor_id', 'aggregate_id', 'server_group_id',
+                       'resource_properties']
+            d.update(self.instance_reservation.to_dict(include=ir_keys))
+
+        if self.floatingip_reservation:
+            fip_keys = ['network_id', 'amount']
+            d.update(self.floatingip_reservation.to_dict(include=fip_keys))
 
         return d
 
@@ -148,6 +155,23 @@ class Event(mb.BlazarBase):
         return super(Event, self).to_dict()
 
 
+class ResourceProperty(mb.BlazarBase):
+    """Defines an resource property by resource type."""
+
+    __tablename__ = 'resource_properties'
+
+    id = _id_column()
+    resource_type = sa.Column(sa.String(255), nullable=False)
+    property_name = sa.Column(sa.String(255), nullable=False)
+    private = sa.Column(sa.Boolean, nullable=False,
+                        server_default=sa.false())
+
+    __table_args__ = (sa.UniqueConstraint('resource_type', 'property_name'),)
+
+    def to_dict(self):
+        return super(ResourceProperty, self).to_dict()
+
+
 class ComputeHostReservation(mb.BlazarBase):
     """Description
 
@@ -180,7 +204,8 @@ class InstanceReservations(mb.BlazarBase):
     memory_mb = sa.Column(sa.Integer, nullable=False)
     disk_gb = sa.Column(sa.Integer, nullable=False)
     amount = sa.Column(sa.Integer, nullable=False)
-    affinity = sa.Column(sa.Boolean, nullable=False)
+    affinity = sa.Column(sa.Boolean, nullable=True)
+    resource_properties = sa.Column(MediumText(), nullable=True)
     flavor_id = sa.Column(sa.String(36), nullable=True)
     aggregate_id = sa.Column(sa.Integer, nullable=True)
     server_group_id = sa.Column(sa.String(36), nullable=True)
@@ -220,6 +245,7 @@ class ComputeHost(mb.BlazarBase):
     memory_mb = sa.Column(sa.Integer, nullable=False)
     local_gb = sa.Column(sa.Integer, nullable=False)
     status = sa.Column(sa.String(13))
+    availability_zone = sa.Column(sa.String(255), nullable=False)
     trust_id = sa.Column(sa.String(36), nullable=False)
     reservable = sa.Column(sa.Boolean, nullable=False,
                            server_default=sa.true())
@@ -243,8 +269,76 @@ class ComputeHostExtraCapability(mb.BlazarBase):
 
     id = _id_column()
     computehost_id = sa.Column(sa.String(36), sa.ForeignKey('computehosts.id'))
-    capability_name = sa.Column(sa.String(64), nullable=False)
+    property_id = sa.Column(sa.String(36),
+                            sa.ForeignKey('resource_properties.id'),
+                            nullable=False)
     capability_value = sa.Column(MediumText(), nullable=False)
 
     def to_dict(self):
         return super(ComputeHostExtraCapability, self).to_dict()
+
+
+# Floating IP
+class FloatingIPReservation(mb.BlazarBase):
+    """Description
+
+    Specifies resources asked by reservation from
+    Floating IP Reservation API.
+    """
+
+    __tablename__ = 'floatingip_reservations'
+
+    id = _id_column()
+    reservation_id = sa.Column(sa.String(36), sa.ForeignKey('reservations.id'))
+    network_id = sa.Column(sa.String(255), nullable=False)
+    amount = sa.Column(sa.Integer, nullable=False)
+    required_fips = relationship('RequiredFloatingIP',
+                                 cascade='all,delete',
+                                 backref='floatingip_reservation',
+                                 lazy='joined')
+
+    def to_dict(self, include=None):
+        d = super(FloatingIPReservation, self).to_dict(include=include)
+        d['required_floatingips'] = [ip['address'] for ip in
+                                     self.required_fips]
+        return d
+
+
+class RequiredFloatingIP(mb.BlazarBase):
+    """A table for a requested Floating IP.
+
+    Keeps an user requested floating IP address in a floating IP reservation.
+    """
+    __tablename__ = 'required_floatingips'
+
+    id = _id_column()
+    address = sa.Column(sa.String(255), nullable=False)
+    floatingip_reservation_id = sa.Column(
+        sa.String(36), sa.ForeignKey('floatingip_reservations.id'))
+
+
+class FloatingIPAllocation(mb.BlazarBase):
+    """Mapping between FloatingIP, FloatingIPReservation and Reservation."""
+
+    __tablename__ = 'floatingip_allocations'
+
+    id = _id_column()
+    floatingip_id = sa.Column(sa.String(36),
+                              sa.ForeignKey('floatingips.id'))
+    reservation_id = sa.Column(sa.String(36),
+                               sa.ForeignKey('reservations.id'))
+
+
+class FloatingIP(mb.BlazarBase):
+    """A table for Floating IP resource."""
+
+    __tablename__ = 'floatingips'
+
+    id = _id_column()
+    floating_network_id = sa.Column(sa.String(255), nullable=False)
+    subnet_id = sa.Column(sa.String(255), nullable=False)
+    floating_ip_address = sa.Column(sa.String(255), nullable=False)
+    reservable = sa.Column(sa.Boolean, nullable=False,
+                           server_default=sa.true())
+
+    __table_args__ = (sa.UniqueConstraint('subnet_id', 'floating_ip_address'),)
