@@ -13,7 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import mock
+from unittest import mock
 
 from oslo_config import cfg
 from oslo_config import fixture as conf_fixture
@@ -60,15 +60,13 @@ class TestTrusts(tests.TestCase):
         self.cfg.config(os_admin_username='admin')
         ctx = self.trusts.create_ctx_from_trust('1')
         fake_ctx_dict = {
-            'auth_token': self.client().auth_token,
+            'auth_token': self.client().session.get_token(),
             'domain': None,
             'global_request_id': self.context.current().global_request_id,
             'is_admin': False,
             'is_admin_project': True,
-            'project': self.client().tenant_id,
+            'project': self.client().session.get_project_id(),
             'project_domain': None,
-            'project_id': self.client().tenant_id,
-            'project_name': 'admin',
             'read_only': False,
             'request_id': ctx.request_id,
             'resource_uuid': None,
@@ -76,10 +74,8 @@ class TestTrusts(tests.TestCase):
             'service_catalog': ctx.service_catalog,
             'show_deleted': False,
             'system_scope': None,
-            'tenant': self.client().tenant_id,
             'user': None,
-            'user_domain': None,
-            'user_id': None}
+            'user_domain': None}
         self.assertDictContainsSubset(fake_ctx_dict, ctx.to_dict())
 
     def test_use_trust_auth_dict(self):
